import sys, pygame, math, random, copy
from pygame.locals import *
from moremath import *
from world import World

HEADERHEIGHT = 0
HEADERTEXTHEIGHT = 40
HEADERFONTWIDTH = 0 # calculated at startup
BORDERWIDTH = 0


#WIDTH, HEIGHT = 1440, 900 #BORDERWIDTH*2+World.WORLDWIDTH, BORDERWIDTH*2+HEADERHEIGHT+World.WORLDWIDTH
WIDTH, HEIGHT = 800, 600

FPS = 25

JUMPSPEED = 3.0
RUNSPEED = 1.5
GRAVITY = 3.0
SPEEDUP = 3

GOODCOLOUR = (50,50,200)
BGCOLOUR = (255,255,255)
BLOCKCOLOUR = (125,125,125)
BADCOLOUR = (100,100,100) #(200,50,50)

TEXTCOLOUR = (0,0,0)
DUDECOLOUR = (255,255,255)
HEADERCOLOUR = (50,50,50)

soundenabled = False #True
soundbeeps, soundjump = [], None

gScreen, gClock = [None]*2
gWorldSurface = None
gTotalTime = 0
gMenuFont = None
gValidKeys = [K_ESCAPE, K_DOWN, K_UP, K_LEFT, K_RIGHT, K_SPACE, K_RETURN, K_r]
gKeys = {}
for key in gValidKeys:
	gKeys[key] = False

# Mode enum
class Mode:
	menu, start, play, pause, dead, highscore, quitt, restart = range(8)
	mode = menu
	
# Action enum
class Action:
	left, right, jump, up, down, select = range(6)

# mode logic
class Menu:
	play, scores, info, quitt = range(4)
	labels = ["play","scores","help","quit"]
	
gMenuItemSelected = Menu.play
gMenuItemSurfaces = []
gMenuItemPositions = [[0,0]]*4
gTitleSurface = None

class MenuMode:
	idle, down, up, selected, info, scores = range(6)

gMenuMode = MenuMode.idle
gMenuScrollSpeed = 100 #ms per item
gMenuScrollCounter = 0

gKeyLock = False #lock escape key from quitting directly after escaping to me

def resetmenu():
	global gMenuItemSelected, gMenuMode, gKeyLock
	gMenuItemSelected = Menu.play
	gMenuMode = MenuMode.idle
	gKeyLock = True

def menu():
	global gMenuItemsSurfaces, gMenuItemSelected, gMenuItemPositions, gMenuScrollCounter, gMenuMode, gKeyLock

	# fill a black rectangle behind menu
	gScreen.fill(BGCOLOUR)
	
	# update mode
	if gMenuMode == MenuMode.idle:	
		if gKeys[K_ESCAPE] and not gKeyLock:
			Mode.mode = Mode.quitt
			return
		elif not gKeys[K_ESCAPE] and gKeyLock:
			gKeyLock = False
		elif gKeys[K_DOWN]:
			gMenuMode = MenuMode.down
			gMenuScrollCounter = gMenuScrollSpeed+1
		elif gKeys[K_UP]:
			gMenuMode = MenuMode.up
			gMenuScrollCounter = gMenuScrollSpeed+1
		elif gKeys[K_SPACE]:
			gMenuMode = MenuMode.selected
	elif gMenuMode == MenuMode.down:
		if not gKeys[K_DOWN]:
			gMenuMode = MenuMode.idle
		gMenuScrollCounter += gClock.get_time()
		if gMenuScrollCounter>gMenuScrollSpeed:
			gMenuScrollCounter = 0
			gMenuItemSelected = (gMenuItemSelected + 1) % len(Menu.labels)
	elif gMenuMode == MenuMode.up:
		if not gKeys[K_UP]:
			gMenuMode = MenuMode.idle
		gMenuScrollCounter += gClock.get_time()
		if gMenuScrollCounter>gMenuScrollSpeed:
			gMenuScrollCounter = 0
			gMenuItemSelected = (gMenuItemSelected - 1) % len(Menu.labels)
	elif gMenuMode == MenuMode.selected:
		if gMenuItemSelected == Menu.quitt:
			Mode.mode = Mode.quitt
			return
		elif gMenuItemSelected == Menu.play:
			Mode.mode = Mode.play
			return
		elif gMenuItemSelected == Menu.info:
			gMenuMode = MenuMode.info
		elif gMenuItemSelected == Menu.scores:
			gMenuMode = MenuMode.scores
	elif gMenuMode == MenuMode.info:
		if gKeys[K_ESCAPE]:
			gMenuMode = MenuMode.idle
			gKeyLock = True
	elif gMenuMode == MenuMode.scores:
		if gKeys[K_ESCAPE]:
			gMenuMode = MenuMode.idle
			gKeyLock = True

	# draw menu
	if not gMenuMode == MenuMode.info and not gMenuMode == MenuMode.scores:
		# draw menu
		ht = gMenuFont.get_height()
		totalmenuheight = len(gMenuItemSurfaces)*ht
		pos = [WIDTH/2,(HEIGHT-totalmenuheight)/2]

		gScreen.blit(gTitleSurface,(pos[0]-gTitleSurface.get_width()/2,pos[1]))
		pos[1] += ht
		
		for i in range(len(gMenuItemSurfaces)):
			item = gMenuItemSurfaces[i]
			w = item.get_width()
			gMenuItemPositions[i] = [pos[0]-w/2,pos[1]]
			gScreen.blit(item,(gMenuItemPositions[i][0],gMenuItemPositions[i][1]))
			pos[1] += ht

		# highlight selected item
		srf = gMenuItemSurfaces[gMenuItemSelected]
		border = 2
		pygame.draw.rect(
			gScreen,
			TEXTCOLOUR,
			pygame.Rect(
				gMenuItemPositions[gMenuItemSelected][0]-border,
				gMenuItemPositions[gMenuItemSelected][1]-border,
				srf.get_width()+2*border,
				srf.get_height()+2*border),
			1)
	elif gMenuMode == MenuMode.info:
		pygame.draw.rect(
			gScreen,
			(255,0,0),
			pygame.Rect(10,10,WIDTH-20,HEIGHT-20))
	elif gMenuMode == MenuMode.scores:
		pygame.draw.rect(
			gScreen,
			(100,100,0),
			pygame.Rect(10,10,WIDTH-20,HEIGHT-20))

gNumberSurfaces = []
def renderDigit(i,pos):
	"assumes i is in [0,9]"
	return gScreen.blit(gNumberSurfaces[i],pos)

def renderInteger(num,pos,paddedwidth=0):
	dlist = [int(x) for x in str(num)] #list of digits
	p = list(pos)
	nzeros = paddedwidth - len(dlist)
	if nzeros > 0:
		while nzeros>0:
			rct = renderDigit(0,p)
			p[0] += rct.width
			nzeros = nzeros-1
		
	for i in dlist:
		rct = renderDigit(i,p)
		p[0] += rct.width

def renderIntegerBounds(num,paddedwidth=0):
	"returns the screen width of the number"
	dlist = [int(x) for x in str(num)] #list of digits
	nzeros = paddedwidth - len(dlist)
	width = 0
	height = 0
	if nzeros > 0:
		width += gNumberSurfaces[0].get_width()*nzeros
		height = max(height,gNumberSurfaces[0].get_height())
	for i in dlist:
		width += gNumberSurfaces[i].get_width()
		height = max(height,gNumberSurfaces[0].get_height())
		
	return (width,height)

# Primary game logic

# The game starts with the player standing on the first block
# Each DT seconds a new block is added to the arena
# Before it is added a rectangle flashes so the player knows where it is appearing
# If the block appears in the same location as the player, the player dies
# The player can jump and move left or right
#   A simple physics and collision detection governs the dynamics of the world
# Blocks disappear after a short time, and no new blocks can appear there
# The game ends if the player dies or reaches the top right corner of the world

# Additional features:
# - blocks have special properties, denoted by their colour
# - white blocks are normal
# - e.g., green blocks enable extra high jumps, blue blocks have no friction
# -       orange blocks are unstable
# - the player can double jump, wall jump, long jump, etc. (feature creep!?)

# COORDINATE SYSTEM
# bottom-left corner of world = (0,0)
# top-right corner = (World.GRIDDIM,World.GRIDDIM)

class PlayMode:
	init, starting, playing, won, lost = range(5)
	mode = init

class Dude:
	# action enums
	standing, walking, jumping, landing = range(4)
	
	#wsz = .4,.66
	#ssz = World.worldToScreenSize(wsz)
	ssz = 18, 25
	wsz = World.screenToWorldSize(ssz)

	pos = [World.GRIDDIM/2.0+0.5-wsz[0]/2,World.GRIDDIM/2.0 + 1] #bottom left point of dude
	vel = [0,0]
	airborn = True
	onblock = None
	collided = False #has he just collided mid-air? affects jumping ability
	facingRight = True  #facing right?

	# frame info

	# current action and frame	
	action = jumping
	currentFrame = None

	internalTimer = 0 #used for timing sprite animation
	walkingFrame = 0

	# imgs
	imgRunL, imgRunR = [], []
	imgStandL, imgStandR = None, None

	RUNRATE = 10 #frame per sec * vel[0]

	@staticmethod
	def load():
		"Loads images and static setup. Called once at startup."
		Dude.imgStandR = pygame.image.load("res\\jstand.png")
		Dude.imgStandR.set_colorkey((255,0,255))
		Dude.imgStandL = pygame.transform.flip(Dude.imgStandR,True,False)


		for i in range(8):
			filename = "res\\jwalk000"+str(i)+".png"
			sfc = pygame.image.load(filename)
			sfc.set_colorkey((255,0,255))
			Dude.imgRunR.append(sfc)
			Dude.imgRunL.append(pygame.transform.flip(sfc,True,False))

		Dude.currentFrame = Dude.imgStandR

	@staticmethod
	def reset():
		Dude.pos = [World.GRIDDIM/2.0+0.5-Dude.wsz[0]/2,World.GRIDDIM/2.0 + 1]
		Dude.vel = [0,0]
		Dude.airborn = True
		Dude.onblock = None
		Dude.collided = False
		Dude.action = Dude.jumping
		Dude.currentFrame = Dude.imgStandR
		Dude.internalTimer = 0
		Dude.walkingFrame = 0
		Dude.facingRight = True

	@staticmethod
	def right():
		return Dude.pos[0] + Dude.wsz[0]
	@staticmethod
	def left():
		return Dude.pos[0]
	@staticmethod
	def top():
		return Dude.pos[1] + Dude.wsz[1]
	@staticmethod
	def bottom():
		return Dude.pos[1]
	
	@staticmethod
	def draw():
		sc = World.worldToScreen(Dude.pos)
		#pygame.draw.rect(gWorldSurface,DUDECOLOUR,pygame.Rect(sc[0],sc[1]-Dude.ssz[1],Dude.ssz[0],Dude.ssz[1]),1)

		gWorldSurface.blit(Dude.currentFrame,(sc[0],sc[1]-Dude.ssz[1]+1))

	@staticmethod
	def jump():
		if not Dude.airborn:
			Dude.vel[1] = JUMPSPEED

	@staticmethod
	def run(hv):
		if not Dude.airborn:
			Dude.vel[0] = hv
		elif not Dude.collided:			
			if Dude.vel[0]>0 and hv < 0:
				Dude.vel[0] *= .8
				Dude.vel[0] = max(Dude.vel[0],0.1)
			elif Dude.vel[0]<0 and hv > 0:
				Dude.vel[0] *= .8
				Dude.vel[0] = min(Dude.vel[0],-0.1)
			else:
				Dude.vel[0] = hv
		
	@staticmethod
	def update():
		dt = gClock.get_time()*SPEEDUP/1000.0
		Dude.internalTimer += dt

		# update pos, vel, action, status, and sprite

		if Dude.vel[0]>0:
			Dude.facingRight = True
		elif Dude.vel[0]<0:
			Dude.facingRight = False
		
		if Dude.airborn:
			Dude.vel[1] -= dt*GRAVITY
			Dude.pos[0] += dt*Dude.vel[0]
			Dude.pos[1] += dt*Dude.vel[1]

			# sides of bounding rectangle
			left, right = Dude.pos[0], Dude.pos[0] + Dude.wsz[0]
			top, bottom = Dude.pos[1]+Dude.wsz[1], Dude.pos[1]
			
			# first with the boundaries of the world	
			if left < 0:
				Dude.pos[0] = 0
				Dude.vel[0] = 0
			elif right > World.GRIDDIM:
				Dude.pos[0] = World.GRIDDIM - Dude.wsz[0]
				Dude.vel[0] = 0
				
			if top > World.GRIDDIM:
				# hit ceiling, thus won game
				Dude.pos[1] = World.GRIDDIM - Dude.wsz[1]
				Dude.vel[1] = 0
				PlayMode.mode = PlayMode.won
			elif bottom < 0:
				# hit floor, thus dead
				Dude.pos[1] = 0
				Dude.vel[1] = 0
				Dude.airborn = False
				PlayMode.mode = PlayMode.lost

			# detect collision with active blocks
			# XXX: We don't need to check all active blocks, as we have the coordinates of the guy!
			# only need to check the block of 4 surrounding the guy!
			irange = range(min(int(Dude.pos[0]),World.GRIDDIM-1),min(int(Dude.pos[0]+Dude.wsz[0])+1,World.GRIDDIM))
			jrange = range(min(int(Dude.pos[1]),World.GRIDDIM-1),min(int(Dude.pos[1]+Dude.wsz[1])+1,World.GRIDDIM))
			# print irange, jrange

			for block in [World.block(i,j) for i in irange for j in jrange if World.block(i,j).gettype()==World.Block.Normal]:			
#			for block in World.activeblocks():
				bleft, bright = block.i, block.i+1
				btop, bbottom = block.j+1, block.j

				if right < bleft: continue
				if left > bright: continue
				if top < bbottom: continue
				if bottom > btop: continue

				# hence a collision has occurred
				side = None

				# calculate dx and dy
				# using the leading edges (based on vel)

				dx, dy = 0, 0
				if Dude.vel[0] < 0:
					dx = bright - left
				elif Dude.vel[0] > 0:
					dx = right - bleft

				if Dude.vel[1] < 0:
					dy = btop - bottom
				elif Dude.vel[1] > 0:
					dy = top - bbottom

				#print "i", dx, dy, Dude.vel

				vellen = math.sqrt(Dude.vel[0]*Dude.vel[0] + Dude.vel[1]*Dude.vel[1])
				velnorm = -Dude.vel[0]/vellen, -Dude.vel[1]/vellen

				t = 0				
				if Dude.vel[0]==0:
					# Note Dude.vel != [0,0], hence
					# t  = ty
					t = dy / math.fabs(velnorm[1])
					if Dude.vel[1]>0:
						side = World.Block.Bottom
					else:
						side = World.Block.Top
				elif Dude.vel[1]==0:
					t = dx / math.fabs(velnorm[0])
					if Dude.vel[0] < 0:
						side = World.Block.Right
					else:
						side = World.Block.Left
				else:
					tx = dx / math.fabs(velnorm[0])
					ty = dy / math.fabs(velnorm[1])
					#print "t",tx,ty
					if tx<ty:						
						t = tx
						if Dude.vel[0]<0:
							side = World.Block.Right
						else:
							side = World.Block.Left
					else:
						t = ty
						if Dude.vel[1]>0:
							side = World.Block.Bottom
						else:
							side = World.Block.Top
							
				epsilon = 0.001 #extra amount to shift dude by	
				disp = (velnorm[0]+sgn(velnorm[0])*epsilon)*t, (velnorm[1]+sgn(velnorm[1])*epsilon)*t
				#print "d", disp
				Dude.pos[0] += disp[0]
				Dude.pos[1] += disp[1]

				# print disp
				if side==World.Block.Top:
					block.setdudeon(True)
					Dude.airborn = False
					Dude.onblock = block
					Dude.vel = [Dude.vel[0],0]
					if not Dude.vel[0]==0:
						Dude.action = Dude.walking
						Dude.internalTimer = 0
						Dude.walkingFrame = 0
						Dude.currentFrame = Dude.imgRunR[0] if Dude.facingRight else Dude.imgRunL[0]
				elif side==World.Block.Bottom:
					Dude.vel = [Dude.vel[0],0]
				elif side==World.Block.Left or side==World.Block.Right:
					Dude.vel = [0,Dude.vel[1]]

				if not side==World.Block.Top:
					Dude.collided = True
				else:
					Dude.collided = False
				
				break		
		else:
			if Dude.vel[1] > 0:
				if soundenabled:
					soundjump.play()
				Dude.airborn = True
				if Dude.onblock:
					Dude.onblock.setdudeon(False)
					Dude.onblock = None
				Dude.action = Dude.jumping
				Dude.internalTimer = 0
				Dude.currentFrame = Dude.imgStandR if Dude.facingRight else Dude.imgStandL
				
			else:
				Dude.vel[0] *= 0.5
				if  math.fabs(Dude.vel[0])<0.01:
					if Dude.onblock.gettype()==World.Block.Empty:
						Dude.airborn = True
						Dude.onblock.setdudeon(False)
						Dude.onblock = None
						Dude.action = Dude.jumping
						Dude.internalTimer = 0
						Dude.currentFrame = Dude.imgStandR if Dude.facingRight else Dude.imgStandL
						return
					else:
						Dude.vel[0] = 0
						Dude.action = Dude.standing
						Dude.currentFrame = Dude.imgStandR if Dude.facingRight else Dude.imgStandL
						Dude.internalTimer = 0
						return
				elif Dude.action == Dude.standing:
					Dude.action = Dude.walking
					Dude.internalTimer = 0
					Dude.walkingFrame = 0
					Dude.currentFrame = Dude.imgRunR[0] if Dude.facingRight else Dude.imgRunL

				Dude.pos[0] += dt*Dude.vel[0]

				if Dude.action == Dude.walking:
					frames = Dude.RUNRATE * math.fabs(Dude.vel[0]) * Dude.internalTimer
					#print frames, Dude.internalTimer
					Dude.walkingFrame += int(frames)
					Dude.walkingFrame %= len(Dude.imgRunR)
					Dude.currentFrame = Dude.imgRunR[Dude.walkingFrame] if Dude.facingRight else Dude.imgRunL[Dude.walkingFrame]
					Dude.internalTimer -= int(frames) / (Dude.RUNRATE * math.fabs(Dude.vel[0]))
					Dude.internalTimer = max(Dude.internalTimer,0)
									
				eps = 0.0001

				# bound to play area
				if Dude.right() > World.GRIDDIM:
					Dude.pos[0] = World.GRIDDIM - Dude.wsz[0] - eps
					return
				elif Dude.left() < 0:
					Dude.pos[0] = eps
					return

				# check collision with block
				# only need to check current row
				row = 0
				if Dude.onblock:
					row = Dude.onblock.j + 1

				for i in range(0,World.GRIDDIM):                              
					bl = World.block(i,row)
					if not bl.gettype()==World.Block.Empty and not bl.gettype()==World.Block.Growing:
						if Dude.left() < bl.i+1 and Dude.right() > bl.i:
							# then a collision has occurred
							# push back dude in opposite velocity direction
	
							if Dude.vel[0] > 0:
								Dude.pos[0] = i - Dude.wsz[0] - eps
							else:
								Dude.pos[0] = i + 1 + eps
							Dude.vel[0] = 0
							Dude.action = Dude.standing
							Dude.currentFrame = Dude.imgStandR if Dude.facingRight else Dude.imgStandL
							Dude.internalTimer = 0
							return
				
				if Dude.onblock:
					ob = Dude.onblock
					if Dude.onblock.gettype()==World.Block.Empty:
						# block must have disappeared, so start falling!
						Dude.airborn = True
						Dude.onblock.setdudeon(False)
						Dude.onblock = None
						Dude.action = Dude.jumping
						Dude.internalTimer = 0
						Dude.currentFrame = Dude.imgStandR if Dude.facingRight else Dude.imgStandL
					elif Dude.pos[0]+Dude.wsz[0] < ob.i:
						bl = World.block(ob.i-1,ob.j)
						if bl.gettype()==World.Block.Empty:
							Dude.airborn = True
							ob.setdudeon(False)
							Dude.onblock = None
							Dude.action = Dude.jumping
							Dude.internalTimer = 0
							Dude.currentFrame = Dude.imgStandR if Dude.facingRight else Dude.imgStandL
						else:
							ob.setdudeon(False)
							Dude.onblock = bl
							bl.setdudeon(True)
					elif Dude.pos[0] > ob.i + 1:
						bl = World.block(ob.i+1,ob.j)
						if bl.gettype()==World.Block.Empty:
							Dude.airborn = True
							ob.setdudeon(False)
							Dude.onblock = None
							Dude.action = Dude.jumping
							Dude.internalTimer = 0
							Dude.currentFrame = Dude.imgStandR if Dude.facingRight else Dude.imgStandL
						else:
							ob.setdudeon(False)
							Dude.onblock = bl
							bl.setdudeon(True)
	
				

headerborder = None
headermargin = BORDERWIDTH


def drawHeader():
	# draw header
	global headerborder #, goodpartImage

	headercolour = colourlerp(GOODCOLOUR,BADCOLOUR,1-float(Dude.pos[1])/World.GRIDDIM)
	
	if headerborder==None:
		headerborder = int((HEADERHEIGHT - HEADERTEXTHEIGHT) / 2)
	pygame.draw.rect(gScreen,headercolour,pygame.Rect(0,0,WIDTH,HEADERHEIGHT))

	renderInteger(int(gTotalTime/1000),(headermargin,headerborder),3)
	renderInteger(World.numactiveblocks(),(WIDTH-HEADERFONTWIDTH*9-headermargin,headerborder),4)
	renderInteger(World.totalnumblocks(),(WIDTH-HEADERFONTWIDTH*4-headermargin,headerborder),4)

	
	pygame.draw.rect(gScreen,BADCOLOUR,pygame.Rect(BORDERWIDTH,HEIGHT-BORDERWIDTH,WIDTH-2*BORDERWIDTH,BORDERWIDTH))
	#pygame.draw.rect(gScreen,GOODCOLOUR,pygame.Rect(BORDERWIDTH,HEADERHEIGHT,WIDTH-2*BORDERWIDTH,BORDERWIDTH))



halfBlockColour = colourlerp(BLOCKCOLOUR,BGCOLOUR,.5)

def drawWorld():
	# draw world
	gWorldSurface.fill(BGCOLOUR,pygame.Rect(0,0,World.WORLDWIDTH,World.WORLDWIDTH))
	
	for block in World.blocks():
		#si, sj = Transform.worldToScreen((block.i,block.j+1))
		#rect = pygame.Rect(si,sj,World.BLOCKWIDTH,World.BLOCKWIDTH)

		pos = block.i*World.BLOCKWIDTH,World.WORLDWIDTH-(block.j+1)*World.BLOCKWIDTH
		rect = pygame.Rect(pos[0]+4,pos[1]+4,World.BLOCKWIDTH-8,World.BLOCKWIDTH-8)
		
		t = 1 - float(block.j) / World.GRIDDIM
		col = colourlerp(BLOCKCOLOUR,(200,200,255),1-t)#GOODCOLOUR,BADCOLOUR,t)
		colour = colourlerp((0,0,0),(255,255,255),float(block.age())/World.DEATHRATE)
				
		if block.gettype()==World.Block.Normal:
			pygame.draw.rect(gWorldSurface,colour,rect)
#			pygame.draw.rect(gWorldSurface,(200,200,200),rect)
##			if block.i>0 and not World.block(block.i-1,block.j).gettype()==World.Block.Normal:
##				pygame.draw.line(gWorldSurface,colour, (pos[0],pos[1]),(pos[0],pos[1]+World.BLOCKWIDTH))
##
##			if block.i<(World.GRIDDIM-1) and not World.block(block.i+1,block.j).gettype()==World.Block.Normal:
##				pygame.draw.line(gWorldSurface,colour, (pos[0]+World.BLOCKWIDTH,pos[1]),(pos[0]+World.BLOCKWIDTH,pos[1]+World.BLOCKWIDTH))
##
##			if block.j>0 and not World.block(block.i,block.j-1).gettype()==World.Block.Normal:
##				pygame.draw.line(gWorldSurface,colour, (pos[0],pos[1]+World.BLOCKWIDTH),(pos[0]+World.BLOCKWIDTH,pos[1]+World.BLOCKWIDTH))
##
##			if block.j<(World.GRIDDIM-1) and not World.block(block.i,block.j+1).gettype()==World.Block.Normal:
##				pygame.draw.line(gWorldSurface,colour, (pos[0],pos[1]),(pos[0]+World.BLOCKWIDTH,pos[1]))
			
			
		elif block.gettype()==World.Block.Growing:
			pygame.draw.rect(gWorldSurface,halfBlockColour,rect,1)
		#elif block.gettype()==World.Block.Empty:
		#	pygame.draw.rect(gWorldSurface,colourlerp((225,225,225),(255,255,255),float(block.j)/World.GRIDDIM),rect)

goodpartImage = None
#badpartImage = None
worldlocation = (WIDTH - World.WORLDWIDTH)/2, (HEIGHT - World.WORLDWIDTH)/2

def drawGame():
	global goodpartImage, badpartImage
	
	#drawHeader()
	drawWorld()
	Dude.draw()

	gScreen.blit(gWorldSurface,worldlocation)

	if goodpartImage==None:
		goodpartImage = pygame.image.load("res\\goodpart.png")
	#	badpartImage = pygame.image.load("res\\badpart.png")

	shftg = 60*(gTotalTime%16000)/16000
	shftb = 60*(gTotalTime%13000)/13000
	
	#gScreen.set_clip(pygame.Rect(worldlocation[0],worldlocation[1]-World.BLOCKWIDTH,World.WORLDWIDTH,World.WORLDWIDTH+2*World.BLOCKWIDTH))
	#for i in range(2+World.WORLDWIDTH/60):
	#	gScreen.blit(goodpartImage,(shftg+worldlocation[0]+(i-1)*60,worldlocation[1]-World.BLOCKWIDTH))

	#for i in range(2+World.WORLDWIDTH/60):
	#	gScreen.blit(badpartImage,(shftb+worldlocation[0]+(i-1)*60,worldlocation[1]+World.WORLDWIDTH))

	#gScreen.set_clip(None)

def backtomenu():
	resetmenu()
	Mode.mode = Mode.menu
	PlayMode.mode = PlayMode.init
	
def play():
	# update mode
	if PlayMode.mode==PlayMode.init:
		gScreen.fill(BGCOLOUR)
		#gScreen.fill(GOODCOLOUR,pygame.Rect(worldlocation[0],worldlocation[1]-World.BLOCKWIDTH,World.WORLDWIDTH,World.BLOCKWIDTH))
		gScreen.fill(BADCOLOUR,pygame.Rect(worldlocation[0],worldlocation[1]+World.WORLDWIDTH,World.WORLDWIDTH,World.BLOCKWIDTH))

		World.reset()
		Dude.reset()
		World.newblock(World.GRIDDIM/2,3,World.Block.Normal)
		PlayMode.mode = PlayMode.starting
		
	elif PlayMode.mode==PlayMode.starting:
		# starting animation
		PlayMode.mode = PlayMode.playing
		
	elif PlayMode.mode==PlayMode.playing:	
		# handle input
		if gKeys[K_ESCAPE]:
			backtomenu()
			return
		elif gKeys[K_r]:
			Mode.mode = Mode.restart
			return		

		if gKeys[K_LEFT]:
			Dude.run(-RUNSPEED)
		elif gKeys[K_RIGHT]:
			Dude.run(RUNSPEED)

		if gKeys[K_UP]:
			Dude.jump()

		# step physics
		Dude.update()

		# check status of game, update() may have changed it
		if not PlayMode.mode==PlayMode.playing:
			return	

		# step world evolution
		World.evolve(gClock.get_time())

		drawGame()
		
	elif PlayMode.mode == PlayMode.won:
		# handle input
		if gKeys[K_ESCAPE]:
			backtomenu()
			return
		elif gKeys[K_r]:
			restart()
			return
		
		# do win sequence
		drawGame()
		
	elif PlayMode.mode == PlayMode.lost:
		# handle input
		if gKeys[K_ESCAPE]:
			backtomenu()
			return
		elif gKeys[K_r]:
			restart()
			return
		
		drawGame()

def restart():
	PlayMode.mode = PlayMode.init
	Mode.mode = Mode.play
	return

def dead():
	None
	# enter dead logic	

#key handler
def keydown(key):
	gKeys[key] = True

def keyup(key):
	gKeys[key] = False

# Main
def main():

	setup()
	
	finished = False	
	# main loop 
	while not finished:
		# handle events		
		for event in pygame.event.get():
			# quit game
			if event.type == QUIT:
				finished = True
				break
			elif event.type == KEYDOWN:
				keydown(event.key)
			elif event.type == KEYUP:
				keyup(event.key)

		#gScreen.fill((0,0,0))
		
		#alphasurf = pygame.Surface((WIDTH,HEIGHT))
		#alphasurf.fill((0,0,0))
		#alphasurf.set_alpha(25)
		#gScreen.blit(alphasurf,(0,0))
		# take a step in current mode
		{Mode.menu: menu,
		 Mode.play: play,
		 Mode.dead: dead,
		 Mode.restart: restart}[Mode.mode]()

		# flip the screen and wait
		pygame.display.flip()
		tick(FPS)
			
		if Mode.mode==Mode.quitt:
			finished = True

	# handle quitting
	pygame.mixer.quit()
	pygame.quit()

def tick(fps):
	global gTotalTime
	gClock.tick(1000.0/fps)
	gTotalTime += gClock.get_time()

def setup():
	"Setup cairo and pygame"
	global gScreen, gWorldSurface, gClock, gTotalTime, gMenuFont, gMenuItemSurfaces, gMenuItemSelected, gNumberSurfaces, HEADERFONTWIDTH, gTitleSurface, soundbeeps, soundjump

	pygame.init()
	pygame.display.set_caption("eden")
	pygame.mouse.set_visible(False)
	gScreen = pygame.display.set_mode((WIDTH,HEIGHT)) #,pygame.locals.HWSURFACE | pygame.locals.FULLSCREEN)

	gWorldSurface = pygame.Surface((World.WORLDWIDTH,World.WORLDWIDTH))
	
	gTotalTime = 0
	gClock = pygame.time.Clock()

	# setup fonts and text
	gMenuFont = pygame.font.Font(pygame.font.get_default_font(),World.WORLDWIDTH/20)

	
	for item in Menu.labels:
		sfc = gMenuFont.render(item,True,TEXTCOLOUR)
		gMenuItemSurfaces.append(sfc)
	gMenuItemSelected = Menu.play

	gTitleSurface = pygame.font.Font(pygame.font.get_default_font(),World.WORLDWIDTH/20).render("eden",True,GOODCOLOUR)

	numberFont = pygame.font.Font(pygame.font.get_default_font(),HEADERTEXTHEIGHT)
	for i in [str(x) for x in range(10)]:
		sfc = numberFont.render(i,True,TEXTCOLOUR)
		gNumberSurfaces.append(sfc)
		HEADERFONTWIDTH = max(HEADERFONTWIDTH,sfc.get_width())

	# sounds
	if pygame.mixer.get_init()==None:
		exit()

	soundjump = pygame.mixer.Sound("C:\\Users\\ben\\Projects\\eden_game\\777_vitriolix_808_kick.wav")
	soundbeeps.append(pygame.mixer.Sound("C:\\Users\\ben\\Projects\\eden_game\\3062_SpeedY_bleep.wav"))
##	arr = pygame.sndarray.array(soundbeeps[0])
##
##	def speedup(s):
##		"returns a sped-up version of arr"
##		newarr = Numeric.zeros((len(arr),len(arr[0])))
##		for i in range(len(newarr)):
##			transformedi = int(i*s)
##			if transformedi>=len(arr):
##				break
##			newarr[i] = arr[transformedi]
##		return newarr
##
##	for i in range(1,World.GRIDDIM):
##		soundbeeps.append(pygame.sndarray.make_sound(speedup(8.0*i/World.GRIDDIM)))

	Dude.load()
	
if __name__ == '__main__': main()
