import random


class World:
	GRIDDIM = 15
	BLOCKWIDTH = 30
	WORLDWIDTH = GRIDDIM*BLOCKWIDTH

	@staticmethod
	def worldToScreen((x,y)):
		"Transforms the world coordinate to screen coordinates"
		return int(x*World.BLOCKWIDTH),int(World.WORLDWIDTH-y*World.BLOCKWIDTH)

	@staticmethod
	def worldToScreenSize((w,h)):
		return (w*World.BLOCKWIDTH,h*World.BLOCKWIDTH)

	@staticmethod
	def screenToWorldSize((sw,sh)):
		return (float(sw) / World.BLOCKWIDTH, float(sh) / World.BLOCKWIDTH)
	
	class Block:
		# type
		Empty, Normal, Growing = range(3)
		# sides
		Left, Right, Top, Bottom = range(4)

		def reset(self):
			self.typ = self.Empty
			self._age = 0
		
		def __init__(self,wc,i,j,t=Empty):
			self.i = i
			self.j = j
			self.typ = t
			self.isdudeon = False
			self.neighbours = []
			self._age = 0
			self.world = wc
			if i>0:
				self.neighbours.append((i-1,j))
			if i<(self.world.GRIDDIM-1):
				self.neighbours.append((i+1,j))
			if j>0:
				self.neighbours.append((i,j-1))
			if j<(self.world.GRIDDIM-1):
				self.neighbours.append((i,j+1))
			return

		def settype(self,t):
			self.typ = t
		def gettype(self):
			return self.typ
		def dudeon(self):
			return self.isdudeon
		def setdudeon(self,don):
			self.isdudeon = don
			return don
		def getneighbours(self):
			return self.neighbours
		def update(self,dt):
			self._age += dt
		def age(self):
			return self._age


	seed = 1234
	rng = random.Random(seed)	
	theblocks = []
	activeBlocks = []
	growingBlocks = []
	totalBlocks = 0
	
	# activeBlocksRects = [] # cached rectangles of active blocks
	currentT = 0
	GROWTHRATE = 200 # in ms
	BIRTHRATE = 1000
	DEATHRATE = 20000

	@classmethod
	def init(cls):
		cls.theblocks = [[World.Block(cls,i,j) for i in range(cls.GRIDDIM)] for j in range(cls.GRIDDIM)]

	@staticmethod
	def reset():
		World.rng = random.Random(World.seed)
		World.clear()

	@staticmethod
	def block(i,j):
		return World.theblocks[j][i]

	@staticmethod
	def blocks():
		for i in range(World.GRIDDIM):
			for j in range(World.GRIDDIM):
				yield World.block(i,j)

	@staticmethod
	def numactiveblocks():
		return len(World.activeBlocks)

	@staticmethod
	def totalnumblocks():
		return World.totalBlocks

	@staticmethod
	def activeblocks():
		for block in World.activeBlocks:
			yield block
	
	@staticmethod
	def newblock(i,j,t):
		bl = World.block(i,j)
		bl.settype(t)
		World.activeBlocks.append(bl)
		World.totalBlocks += 1

	@staticmethod
	def clear():
		World.totalBlocks = 0
		World.activeBlocks = []
		World.growingBlocks = []
		World.currentT = 0
		
		for i in range(World.GRIDDIM):
			for j in range(World.GRIDDIM):
				#World.block(i,j).settype(Block.Empty)
				World.block(i,j).reset()

	@staticmethod
	def evolve(dt):
		"transforms the world according to stochastic growth rules"
		World.currentT += dt
		
		# update growing blocks
		toremove = []
		for blt in World.growingBlocks:			
			blt[1] += dt			
			if blt[1]>World.BIRTHRATE:
				blt[0].settype(World.Block.Normal)
				toremove.append(blt)
				World.activeBlocks.append(blt[0])
				World.totalBlocks += 1
				#if soundenabled:
				#	soundbeeps[0].play() #World.GRIDDIM-1-blt[0].j].play()
				
		for blt in toremove:
			World.growingBlocks.remove(blt)
		toremove = []

		for blk in World.activeBlocks:
			blk.update(dt)
			if blk.age()>World.DEATHRATE:
				toremove.append(blk)
				
		for blk in toremove:
			World.activeBlocks.remove(blk)
			blk.reset()
				
		
		if World.currentT > World.GROWTHRATE:

			# identify squares we can grow into
			accessible_squares = []
			for block in World.blocks():
				nb = [World.block(x[0],x[1]).gettype() for x in block.getneighbours()]
				nbe = nb.count(World.Block.Empty)
				if block.gettype()==World.Block.Empty and nbe==len(nb)-1:
					accessible_squares.append(block)

			if len(accessible_squares)==0:
				print "No more empty spots!"
				exit()
				return

			# select one at random
			nextblock = accessible_squares[World.rng.randint(0,len(accessible_squares)-1)]
			World.growingBlocks.append([nextblock,World.currentT-World.GROWTHRATE])
			nextblock.settype(World.Block.Growing)

			# start growing a new block
##			i,j = random.randint(0,World.GRIDDIM-1), random.randint(0,World.GRIDDIM-1)
##			if World.block(i,j).gettype()==Block.Empty:
##				World.growingBlocks.append([(i,j),World.currentT-World.GROWTHRATE])
##				World.block(i,j).settype(Block.Growing)

			World.currentT -= World.GROWTHRATE

World.init()

